// config.cpp : implementation file
//

#include "stdafx.h"
#include "Termo.h"
#include "config.h"
#include "profile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Cconfig dialog


Cconfig::Cconfig(TEMP_CALIBRATION *_pCalibration,int *_pPort,CWnd* pParent /*=NULL*/)
	: CDialog(Cconfig::IDD, pParent)
{
	//{{AFX_DATA_INIT(Cconfig)
	m_sCurRaw = _T("");
	m_nRaw1 = 0;
	m_nRaw2 = 0;
	m_nTemp1 = 0.0f;
	m_nTemp2 = 0.0f;
	//}}AFX_DATA_INIT

	// Conserve les pointeurs pour la mise a jour en sortie
	pCalibration=_pCalibration;
	pPort=_pPort;
	// Copie les variables pour l'edition
	memcpy(calibration,pCalibration,sizeof(calibration));
	port=*pPort;
	bTimer=FALSE;
}

void Cconfig::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	if (!bTimer)
	{
	//{{AFX_DATA_MAP(Cconfig)
	DDX_Control(pDX, IDC_RADIO1, m_bRadio1);
	DDX_Control(pDX, IDC_RADIO2, m_bRadio2);
	DDX_Control(pDX, IDC_RADIO3, m_bRadio3);
	DDX_Control(pDX, IDC_RADIO4, m_bRadio4);
	DDX_Text(pDX, IDC_ED_RAW1, m_nRaw1);
	DDX_Text(pDX, IDC_ED_RAW2, m_nRaw2);
	DDX_Text(pDX, IDC_ED_TEMP1, m_nTemp1);
	DDX_Text(pDX, IDC_ED_TEMP2, m_nTemp2);
	//}}AFX_DATA_MAP
	}
	else
	{
	DDX_Text(pDX, IDC_CUR_RAW, m_sCurRaw);
	}
}


BEGIN_MESSAGE_MAP(Cconfig, CDialog)
	//{{AFX_MSG_MAP(Cconfig)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_RADIO1, OnRadio1)
	ON_BN_CLICKED(IDC_RADIO2, OnRadio2)
	ON_BN_CLICKED(IDC_RADIO3, OnRadio3)
	ON_BN_CLICKED(IDC_RADIO4, OnRadio4)
	ON_BN_CLICKED(IDC_BTN_APPLY, OnBtnApply)
	ON_EN_CHANGE(IDC_ED_RAW1, OnChangeEdRaw1)
	ON_EN_CHANGE(IDC_ED_RAW2, OnChangeEdRaw2)
	ON_EN_CHANGE(IDC_ED_TEMP1, OnChangeEdTemp1)
	ON_EN_CHANGE(IDC_ED_TEMP2, OnChangeEdTemp2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Cconfig message handlers

BOOL Cconfig::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	switch (port)
	{
		case 0: m_bRadio1.SetCheck(1); break;
		case 1: m_bRadio2.SetCheck(1); break;
		case 2: m_bRadio3.SetCheck(1); break;
		case 3: m_bRadio4.SetCheck(1); break;
	}

	int timerId;
	if ((timerId = SetTimer(0,TIMER_SCALE,NULL))==0)
	{
		// Erreur a tester ici!!
	}

	DataChanged(FALSE);
	SetConfig();
	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void Cconfig::OnOK() 
{
	// Update les variables
	if (ApplyData())
	{
		WriteConfiguration();
		CDialog::OnOK();
	}
}

void Cconfig::OnTimer(UINT nIDEvent) 
{
	// Il faut arreter les fonctions en timer sur la fenetre quand une 
	// autre dialogue prend la main (par ex le control sur la validite des champs "edit")
	// sinon ca plante...
	if (IsWindowEnabled())
	{
		int raw;

		if (ReadTemperature(&raw,NULL,port))
		{
			m_sCurRaw.Format("%4d",raw);
		}
		else
		{
			m_sCurRaw.Format("Non Connect");
		}

		// J'utilise ce flag pour ne modifier que m_sCurRaw dans la dialogue
		// sinon les autres variables se retrouvent rafraichies avec des valeurs
		// fausses. De meme il n'est pas possible de faire un UpdateData() avant
		// car sinon un control sur la validite des champs "edit" est effectue
		// chaque fois que le timer arrive et cela plante completement...

		bTimer=TRUE;

		UpdateData(FALSE);

		CDialog::OnTimer(nIDEvent);
		bTimer =FALSE;
	}
}

void Cconfig::OnRadio1() 
{
	DoRadio(0);
}

void Cconfig::OnRadio2() 
{
	DoRadio(1);
}

void Cconfig::OnRadio3() 
{
	DoRadio(2);
}

void Cconfig::OnRadio4() 
{
	DoRadio(3);
}

void Cconfig::DoRadio(int _port)
{
	if (port!=_port)
	{
		UpdateData();
		GetConfig();
		port=_port;	
		SetConfig();
		UpdateData(FALSE);
		DataChanged(TRUE);	
	}
}

void Cconfig::GetConfig()
{
	calibration[port].raw1 = m_nRaw1;
	calibration[port].raw2 = m_nRaw2;
	calibration[port].temp1 = m_nTemp1;
	calibration[port].temp2 = m_nTemp2;
}

void Cconfig::SetConfig()
{
	m_nRaw1  = calibration[port].raw1 ;
	m_nRaw2  = calibration[port].raw2 ;
	m_nTemp1 = calibration[port].temp1;
	m_nTemp2 = calibration[port].temp2;
}

void Cconfig::OnBtnApply()
{
	ApplyData();
}

BOOL Cconfig::ApplyData()
{
	BOOL bOk=UpdateData();
	if (bOk)
	{
		GetConfig();
		// Update les donnees
		memcpy(pCalibration,calibration,sizeof(calibration));
		*pPort=port;
		DataChanged(FALSE);
	}
	return bOk;
}

void Cconfig::OnChangeEdRaw1() 
{
	DataChanged(TRUE);	
}

void Cconfig::OnChangeEdRaw2() 
{
	DataChanged(TRUE);	
}

void Cconfig::OnChangeEdTemp1() 
{
	DataChanged(TRUE);	
}

void Cconfig::OnChangeEdTemp2() 
{
	DataChanged(TRUE);	
}

void Cconfig::DataChanged(BOOL bDataChanged)
{
	SetItemDisabled(IDC_BTN_APPLY,bDataChanged);
	SetItemDisabled(IDOK,bDataChanged);
}

void Cconfig::SetItemDisabled(int nItem,BOOL bDisabled)
{
	CWnd *pW = GetDlgItem(nItem);
    if (pW)
	{
		DWORD style=0;
		if (bDisabled)
			style^=WS_DISABLED;
		pW->ModifyStyle(style,style^WS_DISABLED,0);
		pW->RedrawWindow();
	}
}
