
#include "stdafx.h"
#include <conio.h>
#include <math.h>
#include "divers.h"

//  =============== Variables ===============

TEMP_CALIBRATION tCalibration[4]=
{
	{803,36,23.9F,100.0F},		// Mes points de mesures...
	{400,36,23.9F,100.0F},		// Valeurs de tests
	{300,36,23.9F,100.0F},
	{200,36,23.9F,100.0F},
};

int currentPort=0;

//  =============== Fonctions ===============

/* -------------------------------------------------
 *  Routine de lecture du port joystick analogique
 *
 *  Parametres:
 *   - un pointeur sur tableau de 4 long qui
 *     dans lequel seront renvoye les valeurs
 *   - masque de bit correspondant aux entrees que
 *     l'on veut lire (ex: pour lire les entrees
 *     1 et 3 le masque sera 5)
 *
 *  Rmq:
 *   - il est aussi possible d'utiliser le bios
 *
 *  Principe de fonctionnement:
 *   - les interruptions sont coupes pour ne pas
 *     pertuber la boucle de lecture.
 *   - le out decharge tous les condensateurs et
 *     les bits des entrees analogiques passe tous
 *     a 1..
 *   - puis on compte pour chaque une des 4 entrees
 *     jusqu'a quel change d'etat (passe a 0)
 *   - le compteur ECX sert de Garde de fou (TimeOut)
 *
 *  Format du registre joystick:
 *
 *  Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0
 *  ------|-------|-------|-------|-------|-------|-------|------
 *  B2 J2 | B1 J2 | B2 J1 | B1 J1 | J2 Y  | J2 X  | J1 Y  | J1 X
 *
 *  Jn  : Joystick 1 ou 2
 *  Bn  : Bouton 1 ou 2: quand le bit est a zero le bouton est appuye
 *  X/Y : entre analogique correspondant a l'axe X ou Y du joystick
 *
 */

void ReadJoystick(int *pData, int mask)
{
	__asm
	{
		cli
		mov	dx,0x201			/* joystick port */
		out	dx,al				/* RAZ port joystick */
		mov ecx,10000
		mov	edi,[pData]
		mov	dword ptr[edi],0
		mov	dword ptr[edi+4],0
		mov	dword ptr[edi+8],0
		mov	dword ptr[edi+12],0

poll_loop:
		in	al,dx				/* lecture du port joystick */
		mov	ebx,eax

		shr	ebx,1              /* test x axis bit */
		adc	dword ptr[edi],0
		shr ebx,1              /* test y axis bit */
		adc dword ptr[edi+4],0
		shr ebx,1              /* test stick 2 x axis bit */
		adc dword ptr[edi+8],0
		shr ebx,1              /* test stick 2 y axis bit */
		adc dword ptr[edi+12],0

		test	mask,eax
		loopnz	poll_loop

		sti
	}
}

/* ---------------------------------------------------
 *  Lecture des entrees numeriques (boutons)
 *  bit 0:bouton 1
 *  bit 1:bouton 2
 *  bit 2:bouton 3
 *  bit 3:bouton 4
 */

int ReadButtons(void)
{
	return (_inp(0x201)>>4)^15;
}

/* ----------------------------------------------------------------------- */
/*
 * Calcul de la temperature en fonction de la valeur lu sur
 * le port joystick:
 *
 * simple interpolation lineaire dans un domaine semi-log...
 * Remarque:la base du log n'a pas d'importance cela change
 *	        simplement les coefficients...
 */

float convertRawToTemp(int raw)
{
	float r1,r2,t1,t2,a,b;

	// Rmq: les valeurs ci-dessous pourraient etre pre-calcule

	r1= log((float)tCalibration[currentPort].raw1);
	r2= log((float)tCalibration[currentPort].raw2);
	t1= tCalibration[currentPort].temp1;
	t2= tCalibration[currentPort].temp2;

	a = (t2-t1)/(r2-r1);
	b = (t1*r2-r1*t2)/(r2-r1);

	return a*log((float)raw)+b;
}


int ReadTemperature(int *pRaw,float *pTemp,int port)
{
	float temp;
	int raw;
	int bOk=FALSE;
#if 0
	// Lecture avec les fonctions Windows
	// attention le calibrage n'est plus le meme alors....
	// NE TIENS PAS COMPTE DE currentPort;
	JOYINFO ji;

	joyGetPos(JOYSTICKID1,&ji);

	raw = ji.wXpos;
#else
	// Lecture avec routine interne

	int joyData[4];
	ReadJoystick(&joyData[0],1<<port);			// Lecture du port joystick pour l'entree courante
	raw = joyData[port];
#endif

	temp = convertRawToTemp(raw);

	if (pRaw)
		*pRaw=raw;
	if (pTemp)
		*pTemp=temp;

	if (raw!=10000)
	{
		bOk=TRUE;
	}

	return bOk;
}